@@ -20,7 +20,7 @@ mergeInto(LibraryManager.library, {
   },
 
 #if ASYNCIFY
-  $Asyncify__deps: ['$runAndAbortIfError', '$callUserCallback',
+  $Asyncify__deps: ['$runAndAbortIfError', '$callUserCallback', '$sigToWasmTypes',
 #if !MINIMAL_RUNTIME
     '$runtimeKeepalivePush', '$runtimeKeepalivePop'
 #endif
@@ -50,6 +50,22 @@ mergeInto(LibraryManager.library, {
     asyncPromiseHandlers: null, // { resolve, reject } pair for when *all* asynchronicity is done
     sleepCallbacks: [], // functions to call every time we sleep
 
+#if ASYNCIFY == 2
+    // TODO: Stack switching support could be implemented without all the
+    //       asyncify infrastructure, perhaps as code on the size instead of
+    //       ifdefs. That might be cleaner, and it would avoid overhead like the
+    //       malloc() asyncify does (we could ifdef it out, but it's even more
+    //       ifdefing).
+
+    // The global suspender object used with the VM's stack switching Promise
+    // API.
+    suspender: null,
+    // The promise that is being suspended on in the VM atm, or null.
+    promise: null,
+    // The function we should call to resolve the promise at the right time.
+    promiseResolve: null,
+#endif
+
     getCallStackId: function(funcName) {
       var id = Asyncify.callStackNameToId[funcName];
       if (id === undefined) {
@@ -60,15 +76,44 @@ mergeInto(LibraryManager.library, {
       return id;
     },
 
-#if ASSERTIONS
     instrumentWasmImports: function(imports) {
-      var ASYNCIFY_IMPORTS = {{{ JSON.stringify(ASYNCIFY_IMPORTS) }}}.map(function(x) {
-        return x.split('.')[1];
-      });
+#if ASYNCIFY_DEBUG
+      err('asyncify instrumenting imports');
+#endif
+#if ASYNCIFY == 2
+      // TODO we could perhaps add an init function and put this there, but
+      //      this should work for now.
+      Asyncify.suspender = new WebAssembly.Suspender();
+#endif
+      var ASYNCIFY_IMPORTS = {{{ JSON.stringify(ASYNCIFY_IMPORTS) }}}.map((x) => x.split('.')[1]);
       for (var x in imports) {
         (function(x) {
           var original = imports[x];
-          if (typeof original === 'function') {
+          var sig = original.sig;
+          if (typeof original == 'function') {
+            var isAsyncifyImport = ASYNCIFY_IMPORTS.indexOf(x) >= 0 ||
+                                   x.startsWith('__asyncjs__');
+#if ASYNCIFY == 2
+            if (isAsyncifyImport) {
+#if ASSERTIONS
+              if (!sig) {
+                throw new Error('Missing __sig for ' + x);
+              }
+#endif
+              var type = sigToWasmTypes(sig, original);
+              // Regardless of the original result type of the function, as it
+              // is now expected to potentially return a Promise, change it to
+              // an externref.
+              type.results = ['externref'];
+#if ASYNCIFY_DEBUG
+              err('asyncify: suspendOnReturnedPromise for', x, original);
+#endif
+              imports[x] = original = Asyncify.suspender.suspendOnReturnedPromise(
+                new WebAssembly.Function(type, original)
+              );
+            }
+#endif
+#if ASSERTIONS && ASYNCIFY != 2 // We cannot apply assertions with stack switching, as the imports must not be modified from suspender.suspendOnReturnedPromise TODO find a way
             imports[x] = function() {
               var originalAsyncifyState = Asyncify.state;
               try {
@@ -76,8 +121,6 @@ mergeInto(LibraryManager.library, {
               } finally {
                 // Only asyncify-declared imports are allowed to change the
                 // state.
-                var isAsyncifyImport = ASYNCIFY_IMPORTS.indexOf(x) >= 0 ||
-                                       x.startsWith('__asyncjs__');
                 // Changing the state from normal to disabled is allowed (in any
                 // function) as that is what shutdown does (and we don't have an
                 // explicit list of shutdown imports).
@@ -95,25 +138,37 @@ mergeInto(LibraryManager.library, {
                   throw new Error('import ' + x + ' was not in ASYNCIFY_IMPORTS, but changed the state');
                 }
               }
-            }
+            };
 #if MAIN_MODULE
             // The dynamic library loader needs to be able to read .sig
             // properties, so that it knows function signatures when it adds
             // them to the table.
-            imports[x].sig = original.sig;
-#endif
+            imports[x].sig = sig;
+#endif // MAIN_MODULE
+#endif // ASSERTIONS
           }
         })(x);
       }
     },
-#endif
 
     instrumentWasmExports: function(exports) {
+#if ASYNCIFY_DEBUG
+      err('asyncify instrumenting exports');
+#endif
       var ret = {};
       for (var x in exports) {
         (function(x) {
           var original = exports[x];
-          if (typeof original === 'function') {
+#if ASYNCIFY == 2
+          // TODO: need a list of all suspending exports.
+          if (x === 'main') {
+#if ASYNCIFY_DEBUG
+            err('asyncify: returnPromiseOnSuspend for', x, original);
+#endif
+            ret[x] = original = Asyncify.suspender.returnPromiseOnSuspend(original);
+          }
+#endif
+          if (typeof original == 'function') {
             ret[x] = function() {
 #if ASYNCIFY_DEBUG >= 2
               err('ASYNCIFY: ' + '  '.repeat(Asyncify.exportCallStack.length) + ' try ' + x);
@@ -132,6 +187,9 @@ mergeInto(LibraryManager.library, {
                 }
               }
             };
+#if MAIN_MODULE
+            ret[x].orig = original;
+#endif
           } else {
             ret[x] = original;
           }
@@ -154,8 +212,10 @@ mergeInto(LibraryManager.library, {
         {{{ runtimeKeepalivePush(); }}}
         Asyncify.state = Asyncify.State.Normal;
         // Keep the runtime alive so that a re-wind can be done later.
-        runAndAbortIfError(Module['_asyncify_stop_unwind']);
-        if (typeof Fibers !== 'undefined') {
+#if ASYNCIFY == 1
+        runAndAbortIfError(_asyncify_stop_unwind);
+#endif
+        if (typeof Fibers != 'undefined') {
           Fibers.trampoline();
         }
       }
@@ -166,7 +226,7 @@ mergeInto(LibraryManager.library, {
       assert(Asyncify.currData, 'Tried to wait for an async operation when none is in progress.');
       assert(!Asyncify.asyncPromiseHandlers, 'Cannot have multiple async operations in flight at once');
 #endif
-      return new Promise(function(resolve, reject) {
+      return new Promise((resolve, reject) => {
         Asyncify.asyncPromiseHandlers = {
           resolve: resolve,
           reject: reject
@@ -203,24 +263,48 @@ mergeInto(LibraryManager.library, {
       {{{ makeSetValue('ptr', C_STRUCTS.asyncify_data_s.rewind_id, 'rewindId', 'i32') }}};
     },
 
+#if RELOCATABLE
+    getDataRewindFunc__deps: [ '$resolveGlobalSymbol' ],
+#endif
     getDataRewindFunc: function(ptr) {
       var id = {{{ makeGetValue('ptr', C_STRUCTS.asyncify_data_s.rewind_id, 'i32') }}};
       var name = Asyncify.callStackIdToName[id];
       var func = Module['asm'][name];
+#if RELOCATABLE
+      // Exported functions in side modules are not listed in `Module["asm"]`,
+      // So we should use `resolveGlobalSymbol` helper function, which is defined in `library_dylink.js`.
+      if (!func) {
+        func = resolveGlobalSymbol(name, false);
+      }
+#endif
       return func;
     },
 
     doRewind: function(ptr) {
+#if ASYNCIFY == 2
+      // Resolve the promise. The VM will resume the wasm on the next event loop
+      // turn.
+      Asyncify.promiseResolve(Asyncify.handleSleepReturnValue);
+      setTimeout(() => {
+        // This timeout happens after the wasm has been resumed; we can stop
+        // artificially keeping the runtime alive at that time.
+        {{{ runtimeKeepalivePop(); }}}
+      });
+#else
       var start = Asyncify.getDataRewindFunc(ptr);
 #if ASYNCIFY_DEBUG
       err('ASYNCIFY: start:', start);
 #endif
-      // Once we have rewound and the stack we no longer need to artificially keep
-      // the runtime alive.
+      // Once we have rewound and the stack we no longer need to artificially
+      // keep the runtime alive.
       {{{ runtimeKeepalivePop(); }}}
       return start();
+#endif
     },
 
+    // This receives a function to call to start the async operation, and
+    // handles everything else for the user of this API. See emscripten_sleep()
+    // and other async methods for simple examples of usage.
     handleSleep: function(startAsync) {
 #if ASSERTIONS
       assert(Asyncify.state !== Asyncify.State.Disabled, 'Asyncify cannot be done during or after the runtime exits');
@@ -236,9 +320,9 @@ mergeInto(LibraryManager.library, {
         // need to do anything.
         var reachedCallback = false;
         var reachedAfterCallback = false;
-        startAsync(function(handleSleepReturnValue) {
+        startAsync((handleSleepReturnValue) => {
 #if ASSERTIONS
-          assert(!handleSleepReturnValue || typeof handleSleepReturnValue === 'number' || typeof handleSleepReturnValue === 'boolean'); // old emterpretify API supported other stuff
+          assert(!handleSleepReturnValue || typeof handleSleepReturnValue == 'number' || typeof handleSleepReturnValue == 'boolean'); // old emterpretify API supported other stuff
 #endif
           if (ABORT) return;
           Asyncify.handleSleepReturnValue = handleSleepReturnValue || 0;
@@ -259,8 +343,10 @@ mergeInto(LibraryManager.library, {
           err('ASYNCIFY: start rewind ' + Asyncify.currData);
 #endif
           Asyncify.state = Asyncify.State.Rewinding;
-          runAndAbortIfError(function() { Module['_asyncify_start_rewind'](Asyncify.currData) });
-          if (typeof Browser !== 'undefined' && Browser.mainLoop.func) {
+#if ASYNCIFY == 1
+          runAndAbortIfError(() => _asyncify_start_rewind(Asyncify.currData));
+#endif
+          if (typeof Browser != 'undefined' && Browser.mainLoop.func) {
             Browser.mainLoop.resume();
           }
           var asyncWasmReturnValue, isError = false;
@@ -308,10 +394,19 @@ mergeInto(LibraryManager.library, {
 #if ASYNCIFY_DEBUG
           err('ASYNCIFY: start unwind ' + Asyncify.currData);
 #endif
-          runAndAbortIfError(function() { Module['_asyncify_start_unwind'](Asyncify.currData) });
-          if (typeof Browser !== 'undefined' && Browser.mainLoop.func) {
+          if (typeof Browser != 'undefined' && Browser.mainLoop.func) {
             Browser.mainLoop.pause();
           }
+#if ASYNCIFY == 2
+          // Return a Promise to get the browser's stack switching logic to run.
+          return Asyncify.promise = new Promise((resolve, reject) => {
+            // Stash the resolve hook so we can call it at the proper time.
+            Asyncify.promiseResolve = resolve;
+            // TODO: handle rejection
+          });
+#else
+          runAndAbortIfError(() => _asyncify_start_unwind(Asyncify.currData));
+#endif
         }
       } else if (Asyncify.state === Asyncify.State.Rewinding) {
         // Stop a resume.
@@ -319,13 +414,13 @@ mergeInto(LibraryManager.library, {
         err('ASYNCIFY: stop rewind');
 #endif
         Asyncify.state = Asyncify.State.Normal;
-        runAndAbortIfError(Module['_asyncify_stop_rewind']);
+#if ASYNCIFY == 1
+        runAndAbortIfError(_asyncify_stop_rewind);
+#endif
         _free(Asyncify.currData);
         Asyncify.currData = null;
         // Call all sleep callbacks now that the sleep-resume is all done.
-        Asyncify.sleepCallbacks.forEach(function(func) {
-          callUserCallback(func);
-        });
+        Asyncify.sleepCallbacks.forEach((func) => callUserCallback(func));
       } else {
         abort('invalid state: ' + Asyncify.state);
       }
@@ -338,23 +433,27 @@ mergeInto(LibraryManager.library, {
     // This is particularly useful for native JS `async` functions where the
     // returned value will "just work" and be passed back to C++.
     handleAsync: function(startAsync) {
-      return Asyncify.handleSleep(function(wakeUp) {
+      return Asyncify.handleSleep((wakeUp) => {
         // TODO: add error handling as a second param when handleSleep implements it.
         startAsync().then(wakeUp);
       });
     },
   },
 
+  emscripten_sleep__sig: 'vi',
   emscripten_sleep__deps: ['$safeSetTimeout'],
   emscripten_sleep: function(ms) {
-    Asyncify.handleSleep(function(wakeUp) {
-      safeSetTimeout(wakeUp, ms);
-    });
+    // emscripten_sleep() does not return a value, but we still need a |return|
+    // here for stack switching support (ASYNCIFY=2). In that mode this function
+    // returns a Promise instead of nothing, and that Promise is what tells the
+    // wasm VM to pause the stack.
+    return Asyncify.handleSleep((wakeUp) => safeSetTimeout(wakeUp, ms));
   },
 
+  emscripten_wget__sig: 'vpp',
   emscripten_wget__deps: ['$Browser', '$PATH_FS', '$FS'],
   emscripten_wget: function(url, file) {
-    Asyncify.handleSleep(function(wakeUp) {
+    return Asyncify.handleSleep((wakeUp) => {
       var _url = UTF8ToString(url);
       var _file = UTF8ToString(file);
       _file = PATH_FS.resolve(FS.cwd(), _file);
@@ -365,20 +464,19 @@ mergeInto(LibraryManager.library, {
         _url, true, true,
         wakeUp,
         wakeUp,
-        undefined, // dontCreateFile
-        undefined, // canOwn
-        function() { // preFinish
-          // if the destination directory does not yet exist, create it
-          FS.mkdirTree(destinationDirectory);
-        }
+        false, // dontCreateFile
+        false, // canOwn
+        // preFinish: if the destination directory does not yet exist, create it
+        () => FS.mkdirTree(destinationDirectory)
       );
     });
   },
 
+  emscripten_wget_data__sig: 'vpppp',
   emscripten_wget_data__deps: ['$asyncLoad', 'malloc'],
   emscripten_wget_data: function(url, pbuffer, pnum, perror) {
-    Asyncify.handleSleep(function(wakeUp) {
-      asyncLoad(UTF8ToString(url), function(byteArray) {
+    return Asyncify.handleSleep((wakeUp) => {
+      asyncLoad(UTF8ToString(url), (byteArray) => {
         // can only allocate the buffer after the wakeUp, not during an asyncing
         var buffer = _malloc(byteArray.length); // must be freed by caller!
         HEAPU8.set(byteArray, buffer);
@@ -386,21 +484,22 @@ mergeInto(LibraryManager.library, {
         {{{ makeSetValue('pnum',  0, 'byteArray.length', 'i32') }}};
         {{{ makeSetValue('perror',  0, '0', 'i32') }}};
         wakeUp();
-      }, function() {
+      }, () => {
         {{{ makeSetValue('perror',  0, '1', 'i32') }}};
         wakeUp();
       }, true /* no need for run dependency, this is async but will not do any prepare etc. step */ );
     });
   },
 
+  emscripten_scan_registers__sig: 'vp',
   emscripten_scan_registers__deps: ['$safeSetTimeout'],
   emscripten_scan_registers: function(func) {
-    Asyncify.handleSleep(function(wakeUp) {
+    return Asyncify.handleSleep((wakeUp) => {
       // We must first unwind, so things are spilled to the stack. Then while
       // we are pausing we do the actual scan. After that we can resume. Note
       // how using a timeout here avoids unbounded call stack growth, which
       // could happen if we tried to scan the stack immediately after unwinding.
-      safeSetTimeout(function() {
+      safeSetTimeout(() => {
         var stackBegin = Asyncify.currData + {{{ C_STRUCTS.asyncify_data_s.__size__ }}};
         var stackEnd = HEAP32[Asyncify.currData >> 2];
         {{{ makeDynCall('vii', 'func') }}}(stackBegin, stackEnd);
@@ -409,8 +508,9 @@ mergeInto(LibraryManager.library, {
     });
   },
 
+  emscripten_lazy_load_code__sig: 'v',
   emscripten_lazy_load_code: function() {
-    Asyncify.handleSleep(function(wakeUp) {
+    return Asyncify.handleSleep((wakeUp) => {
       // Update the expected wasm binary file to be the lazy one.
       wasmBinaryFile += '.lazy.wasm';
       // Add a callback for when all run dependencies are fulfilled, which happens when async wasm loading is done.
@@ -447,7 +547,7 @@ mergeInto(LibraryManager.library, {
       _emscripten_stack_set_limits(stack_base, stack_max);
 
 #if STACK_OVERFLOW_CHECK >= 2
-      Module['___set_stack_limits'](stack_base, stack_max);
+      ___set_stack_limits(stack_base, stack_max);
 #endif
 
       stackRestore({{{ makeGetValue('newFiber', C_STRUCTS.emscripten_fiber_s.stack_ptr,   'i32') }}});
@@ -474,7 +574,7 @@ mergeInto(LibraryManager.library, {
         err('ASYNCIFY/FIBER: start rewind', asyncifyData, '(resuming fiber', newFiber, ')');
 #endif
         Asyncify.state = Asyncify.State.Rewinding;
-        Module['_asyncify_start_rewind'](asyncifyData);
+        _asyncify_start_rewind(asyncifyData);
         Asyncify.doRewind(asyncifyData);
       }
     },
@@ -523,7 +623,7 @@ mergeInto(LibraryManager.library, {
 #if ASYNCIFY_DEBUG
       err('ASYNCIFY/FIBER: start unwind', asyncifyData);
 #endif
-      Module['_asyncify_start_unwind'](asyncifyData);
+      _asyncify_start_unwind(asyncifyData);
 
       var stackTop = stackSave();
       {{{ makeSetValue('oldFiber', C_STRUCTS.emscripten_fiber_s.stack_ptr, 'stackTop', 'i32') }}};
@@ -537,7 +637,7 @@ mergeInto(LibraryManager.library, {
       err('ASYNCIFY/FIBER: stop rewind');
 #endif
       Asyncify.state = Asyncify.State.Normal;
-      Module['_asyncify_stop_rewind']();
+      _asyncify_stop_rewind();
       Asyncify.currData = null;
     }
   },
