@@ -5,11 +5,11 @@
  */
 
 mergeInto(LibraryManager.library, {
-  $NODEFS__deps: ['$FS', '$PATH', '$ERRNO_CODES'],
-  $NODEFS__postset: 'if (ENVIRONMENT_IS_NODE) { var fs = require("fs"); var NODEJS_PATH = require("path"); NODEFS.staticInit(); }',
+  $NODEFS__deps: ['$FS', '$PATH', '$ERRNO_CODES', '$mmapAlloc'],
+  $NODEFS__postset: 'if (ENVIRONMENT_IS_NODE) { requireNodeFS(); NODEFS.staticInit(); }',
   $NODEFS: {
     isWindows: false,
-    staticInit: function() {
+    staticInit: () => {
       NODEFS.isWindows = !!process.platform.match(/^win/);
       var flags = process["binding"]("constants");
       // Node.js 4 compatibility: it has no namespaces for constants
@@ -20,33 +20,34 @@ mergeInto(LibraryManager.library, {
         "{{{ cDefine('O_APPEND') }}}": flags["O_APPEND"],
         "{{{ cDefine('O_CREAT') }}}": flags["O_CREAT"],
         "{{{ cDefine('O_EXCL') }}}": flags["O_EXCL"],
+        "{{{ cDefine('O_NOCTTY') }}}": flags["O_NOCTTY"],
         "{{{ cDefine('O_RDONLY') }}}": flags["O_RDONLY"],
         "{{{ cDefine('O_RDWR') }}}": flags["O_RDWR"],
         "{{{ cDefine('O_DSYNC') }}}": flags["O_SYNC"],
         "{{{ cDefine('O_TRUNC') }}}": flags["O_TRUNC"],
-        "{{{ cDefine('O_WRONLY') }}}": flags["O_WRONLY"]
+        "{{{ cDefine('O_WRONLY') }}}": flags["O_WRONLY"],
+        "{{{ cDefine('O_NOFOLLOW') }}}": flags["O_NOFOLLOW"],
       };
+#if ASSERTIONS
+      // The 0 define must match on both sides, as otherwise we would not
+      // know to add it.
+      assert(NODEFS.flagsForNodeMap["0"] === 0);
+#endif
     },
-    bufferFrom: function (arrayBuffer) {
-      // Node.js < 4.5 compatibility: Buffer.from does not support ArrayBuffer
-      // Buffer.from before 4.5 was just a method inherited from Uint8Array
-      // Buffer.alloc has been added with Buffer.from together, so check it instead
-      return Buffer["alloc"] ? Buffer.from(arrayBuffer) : new Buffer(arrayBuffer);
-    },
-    convertNodeCode: function(e) {
+    convertNodeCode: (e) => {
       var code = e.code;
 #if ASSERTIONS
-      assert(code in ERRNO_CODES);
+      assert(code in ERRNO_CODES, 'unexpected node error code: ' + code + ' (' + e + ')');
 #endif
       return ERRNO_CODES[code];
     },
-    mount: function (mount) {
+    mount: (mount) => {
 #if ASSERTIONS
       assert(ENVIRONMENT_IS_NODE);
 #endif
       return NODEFS.createNode(null, '/', NODEFS.getMode(mount.opts.root), 0);
     },
-    createNode: function (parent, name, mode, dev) {
+    createNode: (parent, name, mode, dev) => {
       if (!FS.isDir(mode) && !FS.isFile(mode) && !FS.isLink(mode)) {
         throw new FS.ErrnoError({{{ cDefine('EINVAL') }}});
       }
@@ -55,7 +56,7 @@ mergeInto(LibraryManager.library, {
       node.stream_ops = NODEFS.stream_ops;
       return node;
     },
-    getMode: function (path) {
+    getMode: (path) => {
       var stat;
       try {
         stat = fs.lstatSync(path);
@@ -70,7 +71,7 @@ mergeInto(LibraryManager.library, {
       }
       return stat.mode;
     },
-    realPath: function (node) {
+    realPath: (node) => {
       var parts = [];
       while (node.parent !== node) {
         parts.push(node.name);
@@ -82,11 +83,12 @@ mergeInto(LibraryManager.library, {
     },
     // This maps the integer permission modes from http://linux.die.net/man/3/open
     // to node.js-specific file open permission strings at http://nodejs.org/api/fs.html#fs_fs_open_path_flags_mode_callback
-    flagsForNode: function(flags) {
-      flags &= ~0x200000 /*O_PATH*/; // Ignore this flag from musl, otherwise node.js fails to open the file.
-      flags &= ~0x800 /*O_NONBLOCK*/; // Ignore this flag from musl, otherwise node.js fails to open the file.
-      flags &= ~0x8000 /*O_LARGEFILE*/; // Ignore this flag from musl, otherwise node.js fails to open the file.
-      flags &= ~0x80000 /*O_CLOEXEC*/; // Some applications may pass it; it makes no sense for a single process.
+    flagsForNode: (flags) => {
+      flags &= ~{{{ cDefine('O_PATH') }}}; // Ignore this flag from musl, otherwise node.js fails to open the file.
+      flags &= ~{{{ cDefine('O_NONBLOCK') }}}; // Ignore this flag from musl, otherwise node.js fails to open the file.
+      flags &= ~{{{ cDefine('O_LARGEFILE') }}}; // Ignore this flag from musl, otherwise node.js fails to open the file.
+      flags &= ~{{{ cDefine('O_CLOEXEC') }}}; // Some applications may pass it; it makes no sense for a single process.
+      flags &= ~{{{ cDefine('O_DIRECTORY') }}}; // Node.js doesn't need this passed in, it errors.
       var newFlags = 0;
       for (var k in NODEFS.flagsForNodeMap) {
         if (flags & k) {
@@ -94,15 +96,13 @@ mergeInto(LibraryManager.library, {
           flags ^= k;
         }
       }
-
-      if (!flags) {
-        return newFlags;
-      } else {
+      if (flags) {
         throw new FS.ErrnoError({{{ cDefine('EINVAL') }}});
       }
+      return newFlags;
     },
     node_ops: {
-      getattr: function(node) {
+      getattr: (node) => {
         var path = NODEFS.realPath(node);
         var stat;
         try {
@@ -135,7 +135,7 @@ mergeInto(LibraryManager.library, {
           blocks: stat.blocks
         };
       },
-      setattr: function(node, attr) {
+      setattr: (node, attr) => {
         var path = NODEFS.realPath(node);
         try {
           if (attr.mode !== undefined) {
@@ -155,12 +155,12 @@ mergeInto(LibraryManager.library, {
           throw new FS.ErrnoError(NODEFS.convertNodeCode(e));
         }
       },
-      lookup: function (parent, name) {
+      lookup: (parent, name) => {
         var path = PATH.join2(NODEFS.realPath(parent), name);
         var mode = NODEFS.getMode(path);
         return NODEFS.createNode(parent, name, mode);
       },
-      mknod: function (parent, name, mode, dev) {
+      mknod: (parent, name, mode, dev) => {
         var node = NODEFS.createNode(parent, name, mode, dev);
         // create the backing node for this in the fs root as well
         var path = NODEFS.realPath(node);
@@ -176,7 +176,7 @@ mergeInto(LibraryManager.library, {
         }
         return node;
       },
-      rename: function (oldNode, newDir, newName) {
+      rename: (oldNode, newDir, newName) => {
         var oldPath = NODEFS.realPath(oldNode);
         var newPath = PATH.join2(NODEFS.realPath(newDir), newName);
         try {
@@ -187,7 +187,7 @@ mergeInto(LibraryManager.library, {
         }
         oldNode.name = newName;
       },
-      unlink: function(parent, name) {
+      unlink: (parent, name) => {
         var path = PATH.join2(NODEFS.realPath(parent), name);
         try {
           fs.unlinkSync(path);
@@ -196,7 +196,7 @@ mergeInto(LibraryManager.library, {
           throw new FS.ErrnoError(NODEFS.convertNodeCode(e));
         }
       },
-      rmdir: function(parent, name) {
+      rmdir: (parent, name) => {
         var path = PATH.join2(NODEFS.realPath(parent), name);
         try {
           fs.rmdirSync(path);
@@ -205,7 +205,7 @@ mergeInto(LibraryManager.library, {
           throw new FS.ErrnoError(NODEFS.convertNodeCode(e));
         }
       },
-      readdir: function(node) {
+      readdir: (node) => {
         var path = NODEFS.realPath(node);
         try {
           return fs.readdirSync(path);
@@ -214,7 +214,7 @@ mergeInto(LibraryManager.library, {
           throw new FS.ErrnoError(NODEFS.convertNodeCode(e));
         }
       },
-      symlink: function(parent, newName, oldPath) {
+      symlink: (parent, newName, oldPath) => {
         var newPath = PATH.join2(NODEFS.realPath(parent), newName);
         try {
           fs.symlinkSync(oldPath, newPath);
@@ -223,20 +223,23 @@ mergeInto(LibraryManager.library, {
           throw new FS.ErrnoError(NODEFS.convertNodeCode(e));
         }
       },
-      readlink: function(node) {
+      readlink: (node) => {
         var path = NODEFS.realPath(node);
         try {
           path = fs.readlinkSync(path);
-          path = NODEJS_PATH.relative(NODEJS_PATH.resolve(node.mount.opts.root), path);
+          path = nodePath.relative(nodePath.resolve(node.mount.opts.root), path);
           return path;
         } catch (e) {
           if (!e.code) throw e;
+          // node under windows can return code 'UNKNOWN' here:
+          // https://github.com/emscripten-core/emscripten/issues/15468
+          if (e.code === 'UNKNOWN') throw new FS.ErrnoError({{{ cDefine('EINVAL') }}});
           throw new FS.ErrnoError(NODEFS.convertNodeCode(e));
         }
       },
     },
     stream_ops: {
-      open: function (stream) {
+      open: (stream) => {
         var path = NODEFS.realPath(stream.node);
         try {
           if (FS.isFile(stream.node.mode)) {
@@ -247,7 +250,7 @@ mergeInto(LibraryManager.library, {
           throw new FS.ErrnoError(NODEFS.convertNodeCode(e));
         }
       },
-      close: function (stream) {
+      close: (stream) => {
         try {
           if (FS.isFile(stream.node.mode) && stream.nfd) {
             fs.closeSync(stream.nfd);
@@ -257,23 +260,23 @@ mergeInto(LibraryManager.library, {
           throw new FS.ErrnoError(NODEFS.convertNodeCode(e));
         }
       },
-      read: function (stream, buffer, offset, length, position) {
+      read: (stream, buffer, offset, length, position) => {
         // Node.js < 6 compatibility: node errors on 0 length reads
         if (length === 0) return 0;
         try {
-          return fs.readSync(stream.nfd, NODEFS.bufferFrom(buffer.buffer), offset, length, position);
+          return fs.readSync(stream.nfd, Buffer.from(buffer.buffer), offset, length, position);
         } catch (e) {
           throw new FS.ErrnoError(NODEFS.convertNodeCode(e));
         }
       },
-      write: function (stream, buffer, offset, length, position) {
+      write: (stream, buffer, offset, length, position) => {
         try {
-          return fs.writeSync(stream.nfd, NODEFS.bufferFrom(buffer.buffer), offset, length, position);
+          return fs.writeSync(stream.nfd, Buffer.from(buffer.buffer), offset, length, position);
         } catch (e) {
           throw new FS.ErrnoError(NODEFS.convertNodeCode(e));
         }
       },
-      llseek: function (stream, offset, whence) {
+      llseek: (stream, offset, whence) => {
         var position = offset;
         if (whence === {{{ cDefine('SEEK_CUR') }}}) {
           position += stream.position;
@@ -294,23 +297,17 @@ mergeInto(LibraryManager.library, {
 
         return position;
       },
-      mmap: function(stream, buffer, offset, length, position, prot, flags) {
+      mmap: (stream, length, position, prot, flags) => {
         if (!FS.isFile(stream.node.mode)) {
           throw new FS.ErrnoError({{{ cDefine('ENODEV') }}});
         }
 
-        // malloc() can lead to growing the heap. If targeting the heap, we need to
-        // re-acquire the heap buffer object in case growth had occurred.
-        var fromHeap = (buffer == HEAPU8);
+        var ptr = mmapAlloc(length);
 
-        var ptr = _malloc(length);
-
-        assert(offset === 0);
-        NODEFS.stream_ops.read(stream, fromHeap ? HEAP8 : buffer, ptr + offset, length, position);
-        
+        NODEFS.stream_ops.read(stream, HEAP8, ptr, length, position);
         return { ptr: ptr, allocated: true };
       },
-      msync: function(stream, buffer, offset, length, mmapFlags) {
+      msync: (stream, buffer, offset, length, mmapFlags) => {
         if (!FS.isFile(stream.node.mode)) {
           throw new FS.ErrnoError({{{ cDefine('ENODEV') }}});
         }
