@@ -48,8 +48,8 @@ mergeInto(LibraryManager.library, {
     createSocket: function(family, type, protocol) {
       type &= ~{{{ cDefine('SOCK_CLOEXEC') | cDefine('SOCK_NONBLOCK') }}}; // Some applications may pass it; it makes no sense for a single process.
       var streaming = type == {{{ cDefine('SOCK_STREAM') }}};
-      if (protocol) {
-        assert(streaming == (protocol == {{{ cDefine('IPPROTO_TCP') }}})); // if SOCK_STREAM, must be tcp
+      if (streaming && protocol && protocol != {{{ cDefine('IPPROTO_TCP') }}}) {
+        throw new FS.ErrnoError({{{ cDefine('EPROTONOSUPPORT') }}}); // if SOCK_STREAM, must be tcp or 0.
       }
 
       // create our internal socket structure
@@ -143,7 +143,7 @@ mergeInto(LibraryManager.library, {
       createPeer: function(sock, addr, port) {
         var ws;
 
-        if (typeof addr === 'object') {
+        if (typeof addr == 'object') {
           ws = addr;
           addr = null;
           port = null;
@@ -204,8 +204,7 @@ mergeInto(LibraryManager.library, {
               // <any space>,<any space> into an Array. Whitespace removal is important for Websockify and ws.
               subProtocols = subProtocols.replace(/^ +| +$/g,"").split(/ *, */);
 
-              // The node ws library API for specifying optional subprotocol is slightly different than the browser's.
-              opts = ENVIRONMENT_IS_NODE ? {'protocol': subProtocols.toString()} : subProtocols;
+              opts = subProtocols;
             }
 
             // some webservers (azure) does not support subprotocol header
@@ -251,7 +250,7 @@ mergeInto(LibraryManager.library, {
         // if this is a bound dgram socket, send the port number first to allow
         // us to override the ephemeral port reported to us by remotePort on the
         // remote end.
-        if (sock.type === {{{ cDefine('SOCK_DGRAM') }}} && typeof sock.sport !== 'undefined') {
+        if (sock.type === {{{ cDefine('SOCK_DGRAM') }}} && typeof sock.sport != 'undefined') {
 #if SOCKET_DEBUG
           out('websocket queuing port message (port ' + sock.sport + ')');
 #endif
@@ -300,7 +299,7 @@ mergeInto(LibraryManager.library, {
         };
 
         function handleMessage(data) {
-          if (typeof data === 'string') {
+          if (typeof data == 'string') {
             var encoder = new TextEncoder(); // should be utf-8
             data = encoder.encode(data); // make a typed array from the string
           } else {
@@ -310,9 +309,8 @@ mergeInto(LibraryManager.library, {
               // as recv/recvmsg will return zero which indicates that a socket
               // has performed a shutdown although the connection has not been disconnected yet.
               return;
-            } else {
-              data = new Uint8Array(data); // make a typed array view on the array buffer
             }
+            data = new Uint8Array(data); // make a typed array view on the array buffer
           }
 
 #if SOCKET_DEBUG
@@ -340,11 +338,11 @@ mergeInto(LibraryManager.library, {
 
         if (ENVIRONMENT_IS_NODE) {
           peer.socket.on('open', handleOpen);
-          peer.socket.on('message', function(data, flags) {
-            if (!flags.binary) {
+          peer.socket.on('message', function(data, isBinary) {
+            if (!isBinary) {
               return;
             }
-            handleMessage((new Uint8Array(data)).buffer);  // copy from node Buffer -> ArrayBuffer
+            handleMessage((new Uint8Array(data)).buffer); // copy from node Buffer -> ArrayBuffer
           });
           peer.socket.on('close', function() {
             Module['websocket'].emit('close', sock.stream.fd);
@@ -444,7 +442,7 @@ mergeInto(LibraryManager.library, {
         return 0;
       },
       bind: function(sock, addr, port) {
-        if (typeof sock.saddr !== 'undefined' || typeof sock.sport !== 'undefined') {
+        if (typeof sock.saddr != 'undefined' || typeof sock.sport != 'undefined') {
           throw new FS.ErrnoError({{{ cDefine('EINVAL') }}});  // already bound
         }
         sock.saddr = addr;
@@ -478,7 +476,7 @@ mergeInto(LibraryManager.library, {
         // }
 
         // early out if we're already connected / in the middle of connecting
-        if (typeof sock.daddr !== 'undefined' && typeof sock.dport !== 'undefined') {
+        if (typeof sock.daddr != 'undefined' && typeof sock.dport != 'undefined') {
           var dest = SOCKFS.websocket_sock_ops.getPeer(sock, sock.daddr, sock.dport);
           if (dest) {
             if (dest.socket.readyState === dest.socket.CONNECTING) {
@@ -541,7 +539,7 @@ mergeInto(LibraryManager.library, {
             Module['websocket'].emit('connection', sock.stream.fd);
           }
         });
-        sock.server.on('closed', function() {
+        sock.server.on('close', function() {
           Module['websocket'].emit('close', sock.stream.fd);
           sock.server = null;
         });
@@ -559,7 +557,7 @@ mergeInto(LibraryManager.library, {
 #endif // ENVIRONMENT_MAY_BE_NODE
       },
       accept: function(listensock) {
-        if (!listensock.server) {
+        if (!listensock.server || !listensock.pending.length) {
           throw new FS.ErrnoError({{{ cDefine('EINVAL') }}});
         }
         var newsock = listensock.pending.shift();
@@ -677,17 +675,14 @@ mergeInto(LibraryManager.library, {
               // if we have a destination address but are not connected, error out
               throw new FS.ErrnoError({{{ cDefine('ENOTCONN') }}});
             }
-            else if (dest.socket.readyState === dest.socket.CLOSING || dest.socket.readyState === dest.socket.CLOSED) {
+            if (dest.socket.readyState === dest.socket.CLOSING || dest.socket.readyState === dest.socket.CLOSED) {
               // return null if the socket has closed
               return null;
             }
-            else {
-              // else, our socket is in a valid state but truly has nothing available
-              throw new FS.ErrnoError({{{ cDefine('EAGAIN') }}});
-            }
-          } else {
+            // else, our socket is in a valid state but truly has nothing available
             throw new FS.ErrnoError({{{ cDefine('EAGAIN') }}});
           }
+          throw new FS.ErrnoError({{{ cDefine('EAGAIN') }}});
         }
 
         // queued.data will be an ArrayBuffer if it's unadulterated, but if it's
@@ -732,25 +727,21 @@ mergeInto(LibraryManager.library, {
    * Passing a NULL callback function to a emscripten_set_socket_*_callback call
    * will deregister the callback registered for that Event.
    */
-#if !MINIMAL_RUNTIME
-  $_setNetworkCallback__deps: ['$runtimeKeepalivePush'],
-#endif
+  $_setNetworkCallback__deps: ['$withStackSave'],
   $_setNetworkCallback: function(event, userData, callback) {
     function _callback(data) {
       try {
         if (event === 'error') {
-          var sp = stackSave();
-          var msg = allocate(intArrayFromString(data[2]), ALLOC_STACK);
-          {{{ makeDynCall('viiii', 'callback') }}}(data[0], data[1], msg, userData);
-          stackRestore(sp);
+          withStackSave(function() {
+            var msg = allocateUTF8OnStack(data[2]);
+            {{{ makeDynCall('viiii', 'callback') }}}(data[0], data[1], msg, userData);
+          });
         } else {
           {{{ makeDynCall('vii', 'callback') }}}(data, userData);
         }
       } catch (e) {
-        if (e instanceof ExitStatus) {
-          return;
-        } else {
-          if (e && typeof e === 'object' && e.stack) err('exception thrown: ' + [e, e.stack]);
+        if (!(e instanceof ExitStatus)) {
+          if (e && typeof e == 'object' && e.stack) err('exception thrown: ' + [e, e.stack]);
           throw e;
         }
       }
