@@ -1,16 +1,21 @@
-// Copyright 2013 The Emscripten Authors.  All rights reserved.
-// Emscripten is available under two separate licenses, the MIT license and the
-// University of Illinois/NCSA Open Source License.  Both these licenses can be
-// found in the LICENSE file.
+/**
+ * @license
+ * Copyright 2013 The Emscripten Authors
+ * SPDX-License-Identifier: MIT
+ */
 
 mergeInto(LibraryManager.library, {
-  $TTY__deps: ['$FS'],
-  $TTY__postset: '__ATINIT__.unshift(function() { TTY.init() });' +
-                 '__ATEXIT__.push(function() { TTY.shutdown() });',
+  $TTY__deps: ['$FS', '$intArrayFromString'],
+#if !MINIMAL_RUNTIME
+  $TTY__postset: function() {
+    addAtInit('TTY.init();');
+    addAtExit('TTY.shutdown();');
+  },
+#endif
   $TTY: {
     ttys: [],
     init: function () {
-      // https://github.com/kripken/emscripten/pull/1555
+      // https://github.com/emscripten-core/emscripten/pull/1555
       // if (ENVIRONMENT_IS_NODE) {
       //   // currently, FS.init does not distinguish if process.stdin is a file or TTY
       //   // device, it always assumes it's a TTY device. because of this, we're forcing
@@ -20,7 +25,7 @@ mergeInto(LibraryManager.library, {
       // }
     },
     shutdown: function() {
-      // https://github.com/kripken/emscripten/pull/1555
+      // https://github.com/emscripten-core/emscripten/pull/1555
       // if (ENVIRONMENT_IS_NODE) {
       //   // inolen: any idea as to why node -e 'process.stdin.read()' wouldn't exit immediately (with process.stdin being a tty)?
       //   // isaacs: because now it's reading from the stream, you've expressed interest in it, so that read() kicks off a _read() which creates a ReadReq operation
@@ -38,7 +43,7 @@ mergeInto(LibraryManager.library, {
       open: function(stream) {
         var tty = TTY.ttys[stream.node.rdev];
         if (!tty) {
-          throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
+          throw new FS.ErrnoError({{{ cDefine('ENODEV') }}});
         }
         stream.tty = tty;
         stream.seekable = false;
@@ -52,7 +57,7 @@ mergeInto(LibraryManager.library, {
       },
       read: function(stream, buffer, offset, length, pos /* ignored */) {
         if (!stream.tty || !stream.tty.ops.get_char) {
-          throw new FS.ErrnoError(ERRNO_CODES.ENXIO);
+          throw new FS.ErrnoError({{{ cDefine('ENXIO') }}});
         }
         var bytesRead = 0;
         for (var i = 0; i < length; i++) {
@@ -60,10 +65,10 @@ mergeInto(LibraryManager.library, {
           try {
             result = stream.tty.ops.get_char(stream.tty);
           } catch (e) {
-            throw new FS.ErrnoError(ERRNO_CODES.EIO);
+            throw new FS.ErrnoError({{{ cDefine('EIO') }}});
           }
           if (result === undefined && bytesRead === 0) {
-            throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
+            throw new FS.ErrnoError({{{ cDefine('EAGAIN') }}});
           }
           if (result === null || result === undefined) break;
           bytesRead++;
@@ -76,21 +81,14 @@ mergeInto(LibraryManager.library, {
       },
       write: function(stream, buffer, offset, length, pos) {
         if (!stream.tty || !stream.tty.ops.put_char) {
-          throw new FS.ErrnoError(ERRNO_CODES.ENXIO);
+          throw new FS.ErrnoError({{{ cDefine('ENXIO') }}});
         }
-        var i = 0;
         try {
-          if (offset === 0 && length === 0) {
-            // musl implements an fflush using a write of a NULL buffer of size 0
-            stream.tty.ops.flush(stream.tty);
-          } else {
-            while (i < length) {
-              stream.tty.ops.put_char(stream.tty, buffer[offset+i]);
-              i++;
-            }
+          for (var i = 0; i < length; i++) {
+            stream.tty.ops.put_char(stream.tty, buffer[offset+i]);
           }
         } catch (e) {
-          throw new FS.ErrnoError(ERRNO_CODES.EIO);
+          throw new FS.ErrnoError({{{ cDefine('EIO') }}});
         }
         if (length) {
           stream.node.timestamp = Date.now();
@@ -106,41 +104,30 @@ mergeInto(LibraryManager.library, {
       get_char: function(tty) {
         if (!tty.input.length) {
           var result = null;
+#if ENVIRONMENT_MAY_BE_NODE
           if (ENVIRONMENT_IS_NODE) {
             // we will read data by chunks of BUFSIZE
             var BUFSIZE = 256;
-            var buf = new Buffer(BUFSIZE);
+            var buf = Buffer.alloc(BUFSIZE);
             var bytesRead = 0;
 
-            var isPosixPlatform = (process.platform != 'win32'); // Node doesn't offer a direct check, so test by exclusion
-
-            var fd = process.stdin.fd;
-            if (isPosixPlatform) {
-              // Linux and Mac cannot use process.stdin.fd (which isn't set up as sync)
-              var usingDevice = false;
-              try {
-                fd = fs.openSync('/dev/stdin', 'r');
-                usingDevice = true;
-              } catch (e) {}
-            }
-
             try {
-              bytesRead = fs.readSync(fd, buf, 0, BUFSIZE, null);
+              bytesRead = fs.readSync(process.stdin.fd, buf, 0, BUFSIZE, -1);
             } catch(e) {
               // Cross-platform differences: on Windows, reading EOF throws an exception, but on other OSes,
               // reading EOF returns 0. Uniformize behavior by treating the EOF exception to return 0.
-              if (e.toString().indexOf('EOF') != -1) bytesRead = 0;
+              if (e.toString().includes('EOF')) bytesRead = 0;
               else throw e;
             }
 
-            if (usingDevice) { fs.closeSync(fd); }
             if (bytesRead > 0) {
               result = buf.slice(0, bytesRead).toString('utf-8');
             } else {
               result = null;
             }
-
-          } else if (typeof window != 'undefined' &&
+          } else
+#endif
+          if (typeof window != 'undefined' &&
             typeof window.prompt == 'function') {
             // Browser.
             result = window.prompt('Input: ');  // returns null on cancel
