@@ -4,7 +4,13 @@
  * SPDX-License-Identifier: MIT
  */
 
+#if SAFE_HEAP
 #include "runtime_safe_heap.js"
+#endif
+
+#if USE_ASAN
+#include "runtime_asan.js"
+#endif
 
 #if ASSERTIONS || SAFE_HEAP
 /** @type {function(*, string=)} */
@@ -43,7 +49,6 @@ if (Module['doWasm2JS']) {
 Module['wasm'] = base64Decode('<<< WASM_BINARY_DATA >>>');
 #endif
 
-#include "runtime_functions.js"
 #include "runtime_strings.js"
 
 var HEAP8, HEAP16, HEAP32, HEAPU8, HEAPU16, HEAPU32, HEAPF32, HEAPF64,
