@@ -137,6 +137,8 @@ def parse_config_file():
     env_var = 'EM_' + key
     env_value = os.environ.get(env_var)
     if env_value is not None:
+      if env_value == '':
+        env_value = None
       globals()[key] = env_value
     elif key in config:
       globals()[key] = config[key]
@@ -167,11 +169,14 @@ def parse_config_file():
   normalize_config_settings()
 
 
-def generate_config(path, first_time=False):
+def generate_config(path):
+  if os.path.exists(path):
+    exit_with_error(f'config file already exists: `{path}`')
+
   # Note: repr is used to ensure the paths are escaped correctly on Windows.
   # The full string is replaced so that the template stays valid Python.
 
-  config_data = utils.read_file(path_from_root('tools/settings_template.py'))
+  config_data = utils.read_file(path_from_root('tools/config_template.py'))
   config_data = config_data.splitlines()[3:] # remove the initial comment
   config_data = '\n'.join(config_data)
   # autodetect some default paths
@@ -179,34 +184,29 @@ def generate_config(path, first_time=False):
   llvm_root = os.path.dirname(which('llvm-dis') or '/usr/bin/llvm-dis')
   config_data = config_data.replace('\'{{{ LLVM_ROOT }}}\'', repr(llvm_root))
 
+  binaryen_root = os.path.dirname(os.path.dirname(which('wasm-opt') or '/usr/local/bin/wasm-opt'))
+  config_data = config_data.replace('\'{{{ BINARYEN_ROOT }}}\'', repr(binaryen_root))
+
   node = which('node') or which('nodejs') or 'node'
   config_data = config_data.replace('\'{{{ NODE }}}\'', repr(node))
 
-  abspath = os.path.abspath(os.path.expanduser(path))
   # write
+  utils.write_file(path, config_data)
 
-  utils.write_file(abspath, config_data)
+  print('''\
+An Emscripten settings file has been generated at:
 
-  if first_time:
-    print('''
-==============================================================================
-Welcome to Emscripten!
-
-This is the first time any of the Emscripten tools has been run.
-
-A settings file has been copied to %s, at absolute path: %s
+  %s
 
 It contains our best guesses for the important paths, which are:
 
   LLVM_ROOT       = %s
+  BINARYEN_ROOT   = %s
   NODE_JS         = %s
   EMSCRIPTEN_ROOT = %s
 
-Please edit the file if any of those are incorrect.
-
-This command will now exit. When you are done editing those paths, re-run it.
-==============================================================================
-''' % (path, abspath, llvm_root, node, __rootpath__), file=sys.stderr)
+Please edit the file if any of those are incorrect.\
+''' % (path, llvm_root, binaryen_root, node, __rootpath__), file=sys.stderr)
 
 
 # Emscripten configuration is done through the --em-config command line option
@@ -226,11 +226,11 @@ embedded_config = path_from_root('.emscripten')
 # For compatibility with `emsdk --embedded` mode also look two levels up.  The
 # layout of the emsdk puts emcc two levels below emsdk.  For example:
 #  - emsdk/upstream/emscripten/emcc
-#  - emsdk/emscipten/1.38.31/emcc
+#  - emsdk/emscripten/1.38.31/emcc
 # However `emsdk --embedded` stores the config file in the emsdk root.
 # Without this check, when emcc is run from within the emsdk in embedded mode
 # and the user forgets to first run `emsdk_env.sh` (which sets EM_CONFIG) emcc
-# will not see any config file at all and fall back to creating a new/emtpy
+# will not see any config file at all and fall back to creating a new/empty
 # one.
 # We could remove this special case if emsdk were to write its embedded config
 # file into the emscripten directory itself.
@@ -254,20 +254,29 @@ elif os.path.exists(emsdk_embedded_config):
 elif os.path.exists(user_home_config):
   EM_CONFIG = user_home_config
 else:
+  # No config file found.  Set EM_CONFIG to a default value
+  # that will get reported in the error below.
   if root_is_writable():
-    generate_config(embedded_config, first_time=True)
+    EM_CONFIG = embedded_config
   else:
-    generate_config(user_home_config, first_time=True)
-  sys.exit(0)
+    EM_CONFIG = user_home_config
 
 # We used to support inline EM_CONFIG.
 if '\n' in EM_CONFIG:
   exit_with_error('Inline EM_CONFIG data no longer supported.  Please use a config file.')
 
 EM_CONFIG = os.path.expanduser(EM_CONFIG)
-logger.debug('emscripten config is located in ' + EM_CONFIG)
+
+# This command line flag needs to work even in the absence of a config file, so we must process it
+# here at script import time (otherwise the error below will trigger).
+if '--generate-config' in sys.argv:
+  generate_config(EM_CONFIG)
+  sys.exit(0)
+
 if not os.path.exists(EM_CONFIG):
-  exit_with_error('emscripten config file not found: ' + EM_CONFIG)
+  exit_with_error(f'config file not found: {EM_CONFIG}.  Please create one by hand or run `emcc --generate-config`')
+
+logger.debug('emscripten config is located in ' + EM_CONFIG)
 
 # Emscripten compiler spawns other processes, which can reimport shared.py, so
 # make sure that those child processes get the same configuration file by
