@@ -5,10 +5,9 @@
 
 import logging
 import os
-import shutil
 
-TAG = '1.70.0'
-HASH = '3ba0180a4a3c20d64727750a3233c82aadba95f265a45052297b955902741edac1befd963400958d6915e5b8d9ade48195eeaf8524f06fdb4cfe43b98677f196'
+TAG = '1.75.0'
+HASH = '8c38be1ebef1b8ada358ad6b7c9ec17f5e0a300e8085db3473a13e19712c95eeb3c3defacd3c53482eb96368987c4b022efa8da2aac2431a154e40153d3c3dcd'
 
 
 def needed(settings):
@@ -16,22 +15,20 @@ def needed(settings):
 
 
 def get(ports, settings, shared):
-  ports.fetch_project('boost_headers', 'https://github.com/emscripten-ports/boost/releases/download/boost-1.70.0/boost-headers-' + TAG + '.zip',
+  ports.fetch_project('boost_headers', 'https://github.com/emscripten-ports/boost/releases/download/boost-1.75.0/boost-headers-' + TAG + '.zip',
                       'boost', sha512hash=HASH)
 
   def create(final):
     logging.info('building port: boost_headers')
-    ports.clear_project_build('boost_headers')
+    build_dir = ports.clear_project_build('boost_headers')
 
     # includes
     source_path_include = os.path.join(ports.get_dir(), 'boost_headers', 'boost')
-    dest_path_include = os.path.join(ports.get_include_dir(), 'boost')
-    shared.try_delete(dest_path_include)
-    shutil.copytree(source_path_include, dest_path_include)
+    ports.install_header_dir(source_path_include, 'boost')
 
     # write out a dummy cpp file, to create an empty library
     # this is needed as emscripted ports expect this, even if it is not used
-    dummy_file = os.path.join(ports.get_build_dir(), 'boost_headers', 'dummy.cpp')
+    dummy_file = os.path.join(build_dir, 'dummy.cpp')
     shared.safe_ensure_dirs(os.path.dirname(dummy_file))
     with open(dummy_file, 'w') as f:
       f.write('static void dummy() {}')
@@ -49,7 +46,7 @@ def get(ports, settings, shared):
 
 
 def clear(ports, settings, shared):
-  shared.Cache.erase_file('libboost_headers.a')
+  shared.Cache.erase_lib('libboost_headers.a')
 
 
 def process_args(ports):
