@@ -24,20 +24,18 @@ def get(ports, settings, shared):
     logging.info('building port: vorbis')
 
     source_path = os.path.join(ports.get_dir(), 'vorbis', 'Vorbis-' + TAG)
-    dest_path = os.path.join(ports.get_build_dir(), 'vorbis')
-
-    shutil.rmtree(dest_path, ignore_errors=True)
+    dest_path = ports.clear_project_build('vorbis')
     shutil.copytree(source_path, dest_path)
 
     ports.build_port(os.path.join(dest_path, 'lib'), final, [os.path.join(dest_path, 'include')],
-                     ['-s', 'USE_OGG=1'], ['psytune', 'barkmel', 'tone', 'misc'])
+                     ['-sUSE_OGG=1'], ['psytune', 'barkmel', 'tone', 'misc'])
     ports.install_header_dir(os.path.join(source_path, 'include', 'vorbis'))
 
   return [shared.Cache.get_lib('libvorbis.a', create)]
 
 
 def clear(ports, settings, shared):
-  shared.Cache.erase_file('libvorbis.a')
+  shared.Cache.erase_lib('libvorbis.a')
 
 
 def process_dependencies(settings):
